<?php
/*
 $Id: sync_common.php,v 1.13 2003/11/20 23:08:27 adeniz Exp $

 Copyright (C) 2003 Baker Research, Ltd.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//
// Configuration.
//
// Default visibility.
$defaultVisibility = 1;
//
// Enable debug output.
//
$debug = true;

/////////////////////////////////////////////////////////////////////////////
// Include PHP distribution files.
/////////////////////////////////////////////////////////////////////////////
$path_pre="../";
$config_path = $path_pre."config.inc.php";
if (is_readable("./$config_path"))
  include_once("./$config_path");
else
  error_exit("$configError:  Unable to open $config_path", $debugVars);

$lib_path = $path_pre."lib";
define ("lib_included","1"); // Needed by following inc files.

// Include database functions.
include_once("$lib_path/db/$db_type.inc.php"); 

// Include encryption functions.
include_once("$lib_path/crypt.inc.php"); 

// set time
$dbTSnull = date("YmdHis", mktime(date("H")+$timezone,date("i"), date("s"),
				  date("m"),date("d"),date("Y")));

/////////////////////////////////////////////////////////////////////////////
// XML tag definitions
/////////////////////////////////////////////////////////////////////////////
$events = "<events>";
$endEvents = "</events>";

$event = "<event>";
$endEvent = "</event>";

$id = "<id>";
$endID = "</id>";

$date = "<date>";
$endDate = "</date>";

$begin = "<begin>";
$endBegin = "</begin>";

$end = "<end>";
$endEnd = "</end>";

$desc = "<desc>";
$endDesc = "</desc>";

$note = "<note>";
$endNote = "</note>";

/////////////////////////////////////////////////////////////////////////////
// Return values
/////////////////////////////////////////////////////////////////////////////

$success = "SUCCESS";
$programError = "PROGRAM_ERROR";
$databaseError = "DATABASE_ERROR";
$dataError = "DATA_ERROR";
$configError = "CONFIG_ERROR";

/////////////////////////////////////////////////////////////////////////////
// Functions
/////////////////////////////////////////////////////////////////////////////

// Return error message and exit.
//
// Inputs--
//   $message--Text message
//   $debugVars--If set, put up a web page with the message
//
////////////////////////////////////////////////////////////////////////////
function error_exit($message, $debugVars) {
  echo "$message";
  if (isset($debugVars))
    echo debug_footer();
  exit;
}

// Replace the PHProjekt function logit and PHP functin die.
// 
// This function logs the message, and then exits.
//
// Inputs--
//   $message--Text message
//
// Global variables--
//   $debugVars--If set, put up a web page with the message
//
////////////////////////////////////////////////////////////////////////////
function logit($message) {
  global $debugVars;

  // Log message.
  logit_no_exit($message);

  // Output remainder of web page and exit.
  error_exit($message, $debugVars);
}

// Replace the PHProjekt function logit.
// 
// This function logs the message.
//
// Inputs--
//   $message--Text message
//
////////////////////////////////////////////////////////////////////////////
function logit_no_exit($message) {

  // Log message.
  openlog("phpsync", LOG_NDELAY|LOG_PID, LOG_USER);
  syslog(LOG_DEBUG, $message);
  closelog();
}

// Check the date format.
//
// See the Protocols manual for the format of the input.
//
// Inputs--
//   $date--Formatted date
//   $debugVars--If set, put up a web page with the message
//
// Returns--
//   Array containing the year, month, and date.
//
//   If there is an error, this function completes the web page and
//   exits the PHP script.
//
////////////////////////////////////////////////////////////////////////////
function check_date_format_exit($date, $debugVars) {
  global $programError;

  // Extract and check the year, month, and date.
  $dates = explode(":", $date);

  // Check the number of elements.
  if (count($dates) != 3)
    error_exit("$programError:  Illegal date specification:  $date",
	$debugVars);

  // Check that the year, month, and date are integers.
  if (!is_numeric($dates[0]) ||
      !is_numeric($dates[1]) ||
      !is_numeric($dates[2]))
    error_exit("$programError:  Illegal date specification:  $date",
	$debugVars);

  // Check the limits of the values.
  if (($dates[0] < 0) || ($dates[0] > 9999) ||
      ($dates[1] < 1) || ($dates[1] > 12) ||
      ($dates[2] < 1) || ($dates[2] > 31))
    error_exit("$programError:  Illegal date specification:  $date",
	$debugVars);

  // Return the array.
  return($dates);
}

// Check the time format.
//
// See the Protocols manual for the format of the input.
//
// Inputs--
//   $time--Formatted date
//   $debugVars--If set, put up a web page with the message
//
// Returns--
//  Array contining the hours and minutes
//
//  If there is an error, this function completes the web page and
//  exits the PHP script.
//
////////////////////////////////////////////////////////////////////////////
function check_time_format_exit($time, $debugVars) {
  global $programError;

  $times = explode(":", $time);

  // Check the number of elements.
  if (count($times) != 2)
    error_exit("$programError:  Illegal time specification:  '$time'",
	$debugVars);

  // Check that the hours and minutes are integers.
  if (!is_numeric($times[0]) || !is_numeric($times[1]))
    error_exit("$programError:  Illegal time specification:  '$time'",
	$debugVars);

  // Check the limits of the values.
  if (($times[0] < 0) || ($times[0] > 24) ||
      ($times[1] < 0) || ($times[1] > 60)) {
    // Allow values of 255:255.
      if($times[0] != 255 && $times[1] != 255 ){
	error_exit("$programError:  Illegal time specification:  '$time'",
	  $debugVars);
      }
  }

  return($times);
}

// Format the date for PHProjekt.
//
// Input--
//  $dates--Array containing the year, month, and date
//
// Returns--
//  String 'YYYY-MM-DD'
//
//////////////////////////////////////////////////////////////////////////
function db_format_date($dates) {
  return(sprintf("%04d-%02d-%02d", $dates[0], $dates[1],$dates[2]));
}

// Format the tiem for PHProjekt.
//
// Input--
//   $times--Array containing the hour and minutes
//
// Returns--
//   String 'HHMM' if there is a time value.
//   String '----' if there is no time value (i.e. the hours:minutes is 255:255
//
//////////////////////////////////////////////////////////////////////////
function db_format_time($times) {
  // If there is no time value, return '----'.
  if( $times[0] == 255 && $times[1] == 255 ) {
    return( '----' );
  } else {
    return(sprintf("%02d%02d", $times[0],$times[1]));
  }
}

// Assemble time WHERE predicate for choosing events in the future.
// See the Protocols manual for the format of the input and the
// purpose of the predicate.
//
// Inputs--
//   $date--Formatted date
//   $time--Formatted date
//   $debugVars--If set, put up a web page with the message
//
// Returns--
//  String containing the WHERE predicate
//
// If there is an error, this function completes the web page and
// exits the PHP script.
//
////////////////////////////////////////////////////////////////////////////
function format_time_predicate_exit($date, $time, $debugVars) {
  global $programError, $dataError;

  // Extract and check the date format.
  $dates = check_date_format_exit($date, $debugVars);

  // Extract and check the hours and minutes.
  $times = check_time_format_exit($time, $debugVars);

  // Get the system date and time.
  $strfDate = strftime("%Y:%m:%d");
  $strfTime = strftime("%H:%M");

  // Load system date and time if it is earlier than the input date.
  if ($strfDate < $date)
    $dates = explode(":", $strfDate);
  if ($strfTime < $time)
    $times = explode(":", $strfTime);

  // Format the date and time for the PHProjekt database.
  $dbDate = db_format_date($dates);
  $dbTime = db_format_time($times);

  // Assemble the WHERE predicate.
  return("(datum > '$dbDate') OR ".
	 "((datum = '$dbDate') AND (ende > '$dbTime'))");
}

// Check the user name and password.  Uses the LDAP server if $ldap global
// variable is set, the PHProjekt database otherwise.
//
// Inputs--
//   $userName--User name
//   $userPW--User password
//   $debugVars--If set, put up a web page with message if there is an error
//
// Returns--
//   The user ID, if successful.
//
//   If there is an error, this function completes the web page and
//   exits the PHP script.
//
////////////////////////////////////////////////////////////////////////////
function check_user_pw($userName, $userPW, $debugVars) {
  global $dataError, $lib_path, $path_pre, $ldap, $loginstring,
    $fetch_uservalues;

  if ($ldap == "1") {
    // Use LDAP.  Load login name and password.
    $loginstring = $userName; // Used by ldap_auth.inc.php.
    $user_pw = $userPW;

    // Authenticate.
    $ldap_path = "sync_ldap_auth.inc.php";
    if (is_readable("$ldap_path"))
      include_once("$ldap_path");
    else
      error_exit("$configError:  Unable to open $ldap_path", $debugVars);

    // Get PHProjekt ID.
    if (isset($fetch_uservalues)) // Set by ldap_auth.inc.php if no error.
      $dbID = $fetch_uservalues;
    else
      error_exit("$dataError:  No unique user '$userName'", $debugVars);
  }
  else {
    // Use the PHProjekt database.
    $result = db_query("SELECT loginname, pw, id FROM users".
		       " WHERE loginname = '$userName'");
    $nRows = 0;
    while ($row = db_fetch_row($result))
      {
	$dbPW = $row[1];
	$dbID = $row[2];
	$nRows++;
      }

    if ($nRows != 1)
      error_exit("$dataError:  No unique user '$userName'", $debugVars);

    // Check the password.
    $encPW = encrypt($userPW, $dbPW);
    if ($dbPW != $encPW)
      error_exit("$dataError:  Incorrect password", $debugVars);
  }

// Return the user ID.
  return($dbID);
}

// Check that there is only one record.
//
// Inputs--
//   $recID--Termine record ID
//   $dbID--User PHProjekt ID
//   $userName--User login name
//   $debugVars--If set, put up a web page with message if there is an error
//
//   If there is an error, this function completes the web page and
//   exits the PHP script.
//
////////////////////////////////////////////////////////////////////////////
function check_unique($recID, $dbID, $userName, $debugVars) {
  global $dataError;

  $query = "SELECT id, von FROM termine WHERE (an = $dbID) AND (id = $recID)";
  $result = db_query($query);
  $nRows = 0;

  // Check that the record exists.
  while ($row = db_fetch_row($result)) {
    $creator = $row[1];
    $nRows++;
  }

  // Check that the record is unique.
  if ($nRows != 1)
    error_exit("$dataError:  There is no unique record '$recID' with user ".
	       "'$userName'", $debugVars);

  // Check that this user created this record, and can therefore modify it.
  //  if ($creator <> $dbID)
  //    error_exit("$dataError:  Attempt to delete a read-only record",
  //	       $debugVars);
}

// Return a string containing an XML element.
//
// Inputs--
//   $data--Contents of the element
//   $tag--Element tag
//
// Returns--
//   String containing the element
//
function xml_element($data, $tag) {
  return("<$tag>$data</$tag>");
}

/////////////////////////////////////////////////////////////////////////////
// Functions for debugging
/////////////////////////////////////////////////////////////////////////////

// Site header for debugging.
function debug_header($title) {
  return ' <HTML>
    <HEAD>
    <TITLE>'.$title.'</TITLE>
    </HEAD>
    <BODY TEXT="#000000" BGCOLOR="#FFFFFF" LINK="#0000EE" VLINK="#551A8B"
    ALINK="#FF0000">

    <FONT FACE="Futura,Arial,Helvetica,Sans">
    <BODY>';
}

// Site footer for debugging.
function debug_footer() {
  return '</BODY></HTML>';
}

//***************************************************************************
// array_recurse & debug
// debug output a var
// by ruediger reinhardt  -  reinhardt@net-attach.de
//***************************************************************************
function array_recurse($item, $spacer = " ")
{
	// increase spacer
	$var_sign = "<b><i><font color='#000080'>Array(" . sizeof($item) . ")</font></i></b>";
	if (strlen($spacer) != 1)
	{
		$spacer .= str_repeat("&nbsp;", strlen(key($item)) + 12);
	}
	else
	{
		$spacer = str_repeat("&nbsp;", strlen(strip_tags($var_sign)));
	}

	echo $var_sign . "<br>\n";

	// walk through the array...
	foreach($item as $key => $value)
	{
		// ...and start another recursion, if the value is an array again...
		if (is_array($value))
		{
        		echo $spacer;
			echo "[<font color='#CA0000'>$key</font>] => ";
			array_recurse($value, $spacer);
		}

		// ...or simply output the var.
		else
		{
			echo $spacer;
        		echo "[<font color='#CA0000'>$key</font>] => ";

                        // special handling for booleans...
        		if (gettype($value) == boolean)
        		{
        			$dsp_value = "false";
        			if ($value)
        			{
        				$dsp_value = "true";
        			}
        			echo "<font color='#408080'> bool(" . $dsp_value . ")</font>";
        		}
        		else
        		{
        			echo "<font color='#408080'>" . gettype($value) . " '" . $value . "'</font>";
        		}
        		echo "<br>\n";
		}
	}
	echo "<br>\n";
}

function debug($var, $dsp_text = "[debug output]")
{
	GLOBAL $debug;

	// debug mode set?
	if ($debug)
	{
		echo "<font face='courier new, courier' size='-1'>\n";
		echo "<hr size='1' noshade><b>$dsp_text: </b><br>\n";

		// if the var is an array, recurse...
		if (is_array($var))
		{
			array_recurse($var);
		}

		// ...or else dump the var.
		else
		{
			var_dump($var);
		}
		echo "<br><hr size='1' noshade>\n";
		echo "</font>\n";
	}
}
?>
